# Copyright (c) 2008 Q2S NTNU, Trondheim, Norway
# All rights reserved.
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License version 2 as
# published by the Free Software Foundation;
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
#
# Port of the Swarm module developed by O. Wittner for ns-2.30.
# Author: Laurent Paquereau <laurent.paquereau@q2s.ntnu.no>
#
# $Header$

#
# Simulator class extensions
#

# Reads <-command,value> tuples
# CEAS-config is the API to configure CEAS in scenario scripts 
# Usage: <Simulator instance> CEAS-config <command> <value>
Simulator instproc CEAS-config args {
    for {} {$args != ""} {set args [lrange $args 2 end]} {
        set key [lindex $args 0]
        set val [lindex $args 1]
        if ![string match {-[A-z]*} $key] {
            puts stderr "Simulator::CEAS-config: Error -\
                         Invalid command '$key'."
            exit 1
        }
        set cmd [string range $key 1 end]
        if {$val == ""} {
            puts stderr "Simulator::CEAS-config: Error -\
                         Command '$cmd' ignored (no value supplied)."
            exit 1
        }
        if [regexp ^log- $cmd] {
            Logger/CEAS init-var [string replace $cmd 0 3] $val
        } else {
            NetworkLayerUnit/CEAS init-var $cmd $val
        }
    }
}

# Prints out current CEAS configuration 
# i.e. all the possible commands for CEAS-config and their value
Simulator instproc dump-CEAS-config { } {
    puts "CEAS-config:"
    NetworkLayerUnit/CEAS dump-vars
    Logger/CEAS dump-vars
}


#
# NetworkLayerUnit/CEAS class
#

# Called by Simulator::CEAS-config (see above)
# Call the static proc '$cmd' in NetworkLayerUnit/CEAS or any of its direct 
# subclasses to set the corresponding static config variable to $val
NetworkLayerUnit/CEAS proc init-var { cmd val } {
    set found 0
    foreach c [NetworkLayerUnit/CEAS info subclass] {
        if {[$c info commands $cmd] != "" } {
            $c $cmd $val
            return
        }
    }
    if { [NetworkLayerUnit/CEAS info commands $cmd] != "" } {
        NetworkLayerUnit/CEAS $cmd $val
        return
    }
    puts stderr "NetworkLayerUnit/CEAS::init-var: Error -\
                 Unknown command (-$cmd)."
    exit 1
}

NetworkLayerUnit/CEAS proc dump-vars { } {
    foreach c [NetworkLayerUnit/CEAS info subclass] {
        if { [$c info commands dump-vars] != "" } {
            $c dump-vars
        }
    }
    puts -nonewline "\t-elite-selection\t\t"
    if {[NetworkLayerUnit/CEAS set config_(elite-selection)] == 0} {
         puts "off" 
    } elseif {[NetworkLayerUnit/CEAS set config_(elite-selection)] == 1} {
         puts "non-explorer"
    } elseif {[NetworkLayerUnit/CEAS set config_(elite-selection)] == 2} {
         puts "all"
    }
    puts -nonewline "\t-routing-loops\t\t\t"
    if {[NetworkLayerUnit/CEAS set config_(routing-loops)] == 0} {
         puts "prevent" 
    } elseif {[NetworkLayerUnit/CEAS set config_(routing-loops)] == 1} {
         puts "allow"
    } elseif {[NetworkLayerUnit/CEAS set config_(routing-loops)] == 2} {
         puts "remove-at-destination"
    }
    puts -nonewline "\t-temperature-approx\t\t"
    if {[NetworkLayerUnit/CEAS set config_(temperature-approx)] == 0} {
         puts "hyperbolic" 
    } elseif {[NetworkLayerUnit/CEAS set config_(temperature-approx)] ==1} {
         puts "linear"
    }
    puts -nonewline "\t-pheromone-approx\t\t"
    if {[NetworkLayerUnit/CEAS set config_(pheromone-approx)] == 0} {
         puts "hyperbolic-2" 
    } elseif {[NetworkLayerUnit/CEAS set config_(pheromone-approx)] ==1} {
         puts "linear-1"
    } elseif {[NetworkLayerUnit/CEAS set config_(pheromone-approx)] ==2} {
         puts "linear-2"
    }
    puts "\t-D\t\t\t\t[NetworkLayerUnit/CEAS set config_(D)]"
    puts "\t-rho\t\t\t\t[NetworkLayerUnit/CEAS set config_(rho)]"
    puts -nonewline "\t-rho-reduction\t\t\t"
    if [NetworkLayerUnit/CEAS set config_(rho-reduction)] {
         puts "on" 
    } else {
         puts "off"
    }
    puts "\t-rho-reduction-factor\t\t[NetworkLayerUnit/CEAS set config_(rho-reduction-factor)]"
}

NetworkLayerUnit/CEAS proc elite-selection { val } {
    set val [string tolower $val]
    if { $val=="off" } {
       NetworkLayerUnit/CEAS set config_(elite-selection) 0
    } elseif { $val=="non-explorer" } {
       NetworkLayerUnit/CEAS set config_(elite-selection) 1
    } elseif { $val=="all" } {
       NetworkLayerUnit/CEAS set config_(elite-selection) 2
    } else {
        puts stderr "NetworkLayerUnit/CEAS::elite-selection: Error -\
                     Invalid value for -elite-selection ($val)."
        exit 1
    }
}

NetworkLayerUnit/CEAS proc routing-loops { val } {
    set val [string tolower $val]
    if { $val=="prevent" } {
        NetworkLayerUnit/CEAS set config_(routing-loops) 0
    } elseif { $val=="allow" } {
        NetworkLayerUnit/CEAS set config_(routing-loops) 1
    } elseif { $val=="remove-at-destination" } {
        NetworkLayerUnit/CEAS set config_(routing-loops) 2
    } else {
        puts stderr "NetworkLayerUnit/CEAS::routing-loops: Error -\
                     Invalid value for -routing-loops ($val)."
        exit 1
    }
}

NetworkLayerUnit/CEAS proc temperature-approx { val } {
    set val [string tolower $val]
    if { $val=="hyperbolic" } {
        NetworkLayerUnit/CEAS set config_(temperature-approx) 0
    } elseif { $val=="linear" } {
        NetworkLayerUnit/CEAS set config_(temperature-approx) 1
    } else {
        puts stderr "NetworkLayerUnit/CEAS::temperature-approx: Error -\
                     Invalid value for -temperature-approx ($val)."
        exit 1
    }
}

NetworkLayerUnit/CEAS proc pheromone-approx { val } {
    set val [string tolower $val]
    if { $val=="hyperbolic-2" } {
        NetworkLayerUnit/CEAS set config_(pheromone-approx) 0
    } elseif { $val=="linear-1" } {
        NetworkLayerUnit/CEAS set config_(pheromone-approx) 1
    } elseif { $val=="linear-2" } {
        NetworkLayerUnit/CEAS set config_(pheromone-approx) 2
    } else {
        puts stderr "NetworkLayerUnit/CEAS::pheromone-approx: Error -\
                     Invalid value for -pheromone-approx ($val)."
        exit 1
    }
}

NetworkLayerUnit/CEAS proc D { val } {
    NetworkLayerUnit/CEAS set config_(D) $val
}

NetworkLayerUnit/CEAS proc rho-reduction { val } {
    set val [string tolower $val]
    if { $val=="on" } {
        NetworkLayerUnit/CEAS set config_(rho-reduction) 1
    } elseif { $val=="off" } {
        NetworkLayerUnit/CEAS set config_(rho-reduction) 0
    } else {
        puts stderr "NetworkLayerUnit/CEAS::cost: Error - Invalid value for\
                     -rho-reduction ($val)."
        exit 1
    }
}

NetworkLayerUnit/CEAS proc rho { val } {
    NetworkLayerUnit/CEAS set config_(rho) $val
}

NetworkLayerUnit/CEAS proc rho-reduction-factor { val } {
    NetworkLayerUnit/CEAS set config_(rho-reduction-factor) $val
}


#
# Logger/CEAS class
#

# Called by Simulator::CEAS-config (see above)
# Call the static proc '$cmd' in Logger/CEAS or any of its direct 
# subclasses to set the corresponding static config variable to $val
Logger/CEAS proc init-var { cmd val } {
    set found 0
    foreach c [Logger/CEAS info subclass] {
        if {[$c info commands $cmd] != "" } {
            $c $cmd $val
            return
        }
    }
    if { [Logger/CEAS info commands $cmd] != "" } {
        Logger/CEAS $cmd $val
        return
    }
    puts stderr "Logger/CEAS::init-var: Error - Unknown command (-log-$cmd)."
    exit 1
}

Logger/CEAS proc dump-vars { } {
    foreach c [Logger/CEAS info subclass] {
        if { [$c info commands dump-vars] != "" } {
            $c dump-vars
        }
    }
    puts "\t-log-bin-size\t\t\t[Logger/CEAS set config_(bin-size)]"
    puts "\t-log-sample-freq\t\t[Logger/CEAS set config_(sample-freq)]"
    puts -nonewline "\t-log-cost\t\t\t"
    if [Logger/CEAS set config_(cost)] {
         puts "on" 
    } else {
         puts "off"
    }
    puts -nonewline "\t-log-cost-all\t\t\t"
    if [Logger/CEAS set config_(cost-all)] {
         puts "on" 
    } else {
         puts "off"
    }
    puts -nonewline "\t-log-cost-best\t\t\t"
    if [Logger/CEAS set config_(cost-best)] {
         puts "on" 
    } else {
         puts "off"
    }
    puts -nonewline "\t-log-path\t\t\t"
    if [Logger/CEAS set config_(path)] {
         puts "on" 
    } else {
         puts "off"
    }
    puts -nonewline "\t-log-path-all\t\t\t"
    if [Logger/CEAS set config_(path-all)] {
         puts "on" 
    } else {
         puts "off"
    }
    puts -nonewline "\t-log-delay\t\t\t"
    if [Logger/CEAS set config_(delay)] {
         puts "on" 
    } else {
         puts "off"
    }
    puts -nonewline "\t-log-elite-gamma\t\t"
    if [Logger/CEAS set config_(elite-gamma)] {
         puts "on" 
    } else {
         puts "off"
    }
   puts -nonewline "\t-log-total-gamma\t\t"
    if [Logger/CEAS set config_(total-gamma)] {
         puts "on" 
    } else {
         puts "off"
    }
   puts -nonewline "\t-log-total-temperature-vars\t"
    if [Logger/CEAS set config_(total-temperature-vars)] {
         puts "on" 
    } else {
         puts "off"
    }
    puts -nonewline "\t-log-gamma-diff\t\t\t"
    if [Logger/CEAS set config_(gamma-diff)] {
         puts "on" 
    } else {
         puts "off"
    }
    puts -nonewline "\t-log-gamma-rel\t\t\t"
    if [Logger/CEAS set config_(gamma-rel)] {
         puts "on" 
    } else {
         puts "off"
    }
    puts -nonewline "\t-log-elite-selection-level\t"
    if [Logger/CEAS set config_(elite-selection-level)] {
         puts "on" 
    } else {
         puts "off"
    }
    puts -nonewline "\t-log-rho\t\t\t"
    if [Logger/CEAS set config_(rho)] {
         puts "on" 
    } else {
         puts "off"
    }
    puts -nonewline "\t-log-forwardcount\t\t"
    if [Logger/CEAS set config_(forwardcount)] {
         puts "on" 
    } else {
         puts "off"
    }
    puts -nonewline "\t-log-backtrackcount\t\t"
    if [Logger/CEAS set config_(backtrackcount)] {
         puts "on" 
    } else {
         puts "off"
    }
    puts -nonewline "\t-log-gamma-calc-count\t\t"
    if [Logger/CEAS set config_(gamma-calc-count)] {
         puts "on" 
    } else {
         puts "off"
    }
    puts -nonewline "\t-log-pheromones\t\t\t"
    if [Logger/CEAS set config_(pheromones)] {
         puts "on" 
    } else {
         puts "off"
    }
    puts -nonewline "\t-log-pheromones-id\t\t"
    if {[Logger/CEAS set config_(pheromones-id)]==""} {
         puts "\"\" (all)" 
    } else {
         puts [Logger/CEAS set config_(pheromones-id)]
    }
    puts -nonewline "\t-log-pheromones-at\t\t"
    if {[Logger/CEAS set config_(pheromones-at)]==""} {
         puts "\"\" (all)" 
    } else {
         puts [Logger/CEAS set config_(pheromones-at)]
    }
    puts -nonewline "\t-log-only\t\t\t"
    if {[Logger/CEAS set config_(only)]==""} {
         puts "\"\" (all)" 
    } else {
         puts [Logger/CEAS set config_(only)]
    }
}

Logger/CEAS proc bin-size { val } {
    Logger/CEAS set config_(bin-size) $val
}

Logger/CEAS proc sample-freq { val } {
    Logger/CEAS set config_(sample-freq) $val
}

Logger/CEAS proc cost { val } {
    set val [string tolower $val]
    if { $val=="on" } {
        Logger/CEAS set config_(cost) 1
    } elseif { $val=="off" } {
        Logger/CEAS set config_(cost) 0
    } else {
        puts stderr "Logger/CEAS::cost: Error - Invalid value for -cost ($val)."
        exit 1
    }
}

Logger/CEAS proc cost-all { val } {
    set val [string tolower $val]
    if { $val=="on" } {
        Logger/CEAS set config_(cost-all) 1
    } elseif { $val=="off" } {
        Logger/CEAS set config_(cost-all) 0
    } else {
        puts stderr "Logger/CEAS::cost-all: Error -\
                     Invalid value for -cost-all ($val)."
        exit 1
    }
}

Logger/CEAS proc cost-best { val } {
    set val [string tolower $val]
    if { $val=="on" } {
        Logger/CEAS set config_(cost-best) 1
    } elseif { $val=="off" } {
        Logger/CEAS set config_(cost-best) 0
    } else {
        puts stderr "Logger/CEAS::cost-best: Error -\
                     Invalid value for -cost-best ($val)."
        exit 1
    }
}

Logger/CEAS proc path { val } {
    set val [string tolower $val]
    if { $val=="on" } {
        Logger/CEAS set config_(path) 1
    } elseif { $val=="off" } {
        Logger/CEAS set config_(path) 0
    } else {
        puts stderr "Logger/CEAS::path: Error - Invalid value for -path ($val)."
        exit 1
    }
}

Logger/CEAS proc path-all { val } {
    set val [string tolower $val]
    if { $val=="on" } {
        Logger/CEAS set config_(path-all) 1
    } elseif { $val=="off" } {
        Logger/CEAS set config_(path-all) 0
    } else {
        puts stderr "Logger/CEAS::path-all: Error -\
                     Invalid value for -path-all ($val)."
        exit 1
    }
}

Logger/CEAS proc delay { val } {
    set val [string tolower $val]
    if { $val=="on" } {
        Logger/CEAS set config_(delay) 1
    } elseif { $val=="off" } {
        Logger/CEAS set config_(delay) 0
    } else {
        puts stderr "Logger/CEAS::delay: Error -\
                     Invalid value for -delay ($val)."
        exit 1
    }
}

Logger/CEAS proc elite-gamma { val } {
    set val [string tolower $val]
    if { $val=="on" } {
        Logger/CEAS set config_(elite-gamma) 1
    } elseif { $val=="off" } {
        Logger/CEAS set config_(elite-gamma) 0
    } else {
        puts stderr "Logger/CEAS::elite-gamma: Error -\
                     Invalid value for -elite-gamma ($val)."
        exit 1
    }
}

Logger/CEAS proc total-gamma { val } {
    set val [string tolower $val]
    if { $val=="on" } {
        Logger/CEAS set config_(total-gamma) 1
    } elseif { $val=="off" } {
        Logger/CEAS set config_(total-gamma) 0
    } else {
        puts stderr "Logger/CEAS::total-gamma: Error -\
                     Invalid value for -total-gamma ($val)."
        exit 1
    }
}

Logger/CEAS proc total-temperature-vars { val } {
    set val [string tolower $val]
    if { $val=="on" } {
        Logger/CEAS set config_(total-temperature-vars) 1
    } elseif { $val=="off" } {
        Logger/CEAS set config_(total-temperature-vars) 0
    } else {
        puts stderr "Logger/CEAS::total-temperature-vars: Error -\
                     Invalid value for -total-temperature-vars ($val)."
        exit 1
    }
}

Logger/CEAS proc gamma-diff { val } {
    set val [string tolower $val]
    if { $val=="on" } {
        Logger/CEAS set config_(gamma-diff) 1
    } elseif { $val=="off" } {
        Logger/CEAS set config_(gamma-diff) 0
    } else {
        puts stderr "Logger/CEAS::gamma-diff: Error -\
                     Invalid value for -gamma-diff ($val)."
        exit 1
    }
}

Logger/CEAS proc gamma-rel { val } {
    set val [string tolower $val]
    if { $val=="on" } {
        Logger/CEAS set config_(gamma-rel) 1
    } elseif { $val=="off" } {
        Logger/CEAS set config_(gamma-rel) 0
    } else {
        puts stderr "Logger/CEAS::gamma-rel: Error -\
                     Invalid value for -gamma-rel ($val)."
        exit 1
    }
}

Logger/CEAS proc elite-selection-level { val } {
    set val [string tolower $val]
    if { $val=="on" } {
        Logger/CEAS set config_(elite-selection-level) 1
    } elseif { $val=="off" } {
        Logger/CEAS set config_(elite-selection-level) 0
    } else {
        puts stderr "Logger/CEAS::elite-selection-level: Error -\
                     Invalid value for -elite-selection-level ($val)."
        exit 1
    }
}

Logger/CEAS proc rho { val } {
    set val [string tolower $val]
    if { $val=="on" } {
        Logger/CEAS set config_(rho) 1
    } elseif { $val=="off" } {
        Logger/CEAS set config_(rho) 0
    } else {
        puts stderr "Logger/CEAS::rho: Error - Invalid value for -rho ($val)."
        exit 1
    }
}

Logger/CEAS proc backtrackcount { val } {
    set val [string tolower $val]
    if { $val=="on" } {
        Logger/CEAS set config_(backtrackcount) 1
    } elseif { $val=="off" } {
        Logger/CEAS set config_(backtrackcount) 0
    } else {
        puts stderr "Logger/CEAS::backtrackcount: Error -\
                     Invalid value for -backtrackcount ($val)."
        exit 1
    }
}

Logger/CEAS proc forwardcount { val } {
    set val [string tolower $val]
    if { $val=="on" } {
        Logger/CEAS set config_(forwardcount) 1
    } elseif { $val=="off" } {
        Logger/CEAS set config_(forwardcount) 0
    } else {
        puts stderr "Logger/CEAS::forwardcount: Error -\
                     Invalid value for -forwardcount ($val)."
        exit 1
    }
}

Logger/CEAS proc pheromones { val } {
    set val [string tolower $val]
    if { $val=="on" } {
        Logger/CEAS set config_(pheromones) 1
    } elseif { $val=="off" } {
        Logger/CEAS set config_(pheromones) 0
    } else {
        puts stderr "Logger/CEAS::pheromones: Error -\
                     Invalid value for -pheromones ($val)."
        exit 1
    }
}

Logger/CEAS proc gamma-calc-count { val } {
    set val [string tolower $val]
    if { $val=="on" } {
        Logger/CEAS set config_(gamma-calc-count) 1
    } elseif { $val=="off" } {
        Logger/CEAS set config_(gamma-calc-count) 0
    } else {
        puts stderr "Logger/CEAS::gamma-calc-count: Error -\
                     Invalid value for -gamma-calc-count ($val)."
        exit 1
    }
}

Logger/CEAS proc pheromones-id { val } {
    Logger/CEAS set config_(pheromones-id) $val
}

Logger/CEAS proc only { val } {
    Logger/CEAS set config_(only) $val
}

Logger/CEAS proc pheromones-at { val } {
    Logger/CEAS set config_(pheromones-at) $val
}
