/*
 * Copyright (c) 2008 Q2S NTNU, Trondheim, Norway
 * All rights reserved.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation;
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 * Port of the Swarm module developed by O. Wittner for ns-2.30.
 * Author: Laurent Paquereau <laurent.paquereau@q2s.ntnu.no>
 *
 * $Header$
 */
 
#ifndef _ceas_routing_unit_ib_h
#define _ceas_routing_unit_ib_h

#include <map>

// Forward declarations
template<class _Key,class _PathItemData> class CEASRoutingUnit;
template<class __Key,class __Tp,class __PathItemData> class CEASRoutingUnitIB;

//! CEAS Routing Unit Information Base template class
template<class __Key,class __Tp,class __PathItemData> class CEASRoutingUnitIB<__Key,__Tp*,__PathItemData> {
  public:
    //! Constructor
    CEASRoutingUnitIB(CEASRoutingUnit<__Key,__PathItemData>* routingUnit)
      : routingUnit_(routingUnit){}
    virtual ~CEASRoutingUnitIB();
    virtual __Tp* addEntry(const __Key& key);
    __Tp* findEntry(const __Key& key);
    __Tp* getEntry(const __Key& key);
    inline __Tp* operator[] (const __Key& key) {return getEntry(key);}
  protected:
    map<__Key,__Tp*> base_; //!< the base itself
    CEASRoutingUnit<__Key,__PathItemData>* routingUnit_;
    
};

//! Destructor
/*! Clear the base and delete all the entries */
template<class __Key,class __Tp,class __PathItemData>
CEASRoutingUnitIB<__Key,__Tp*,__PathItemData>::~CEASRoutingUnitIB()
{
    while (!base_.empty())
      {
        typename map<__Key,__Tp*>::iterator i = base_.begin();
        __Tp* entry =
          static_cast<__Tp*>(i->second);
        delete entry;
        base_.erase(i);
      }
}

//! Create new entry for the given key and add it to the base.
/*! If there is already an entry for this key, replace it. */
template<class __Key,class __Tp,class __PathItemData> __Tp*
CEASRoutingUnitIB<__Key,__Tp*,__PathItemData>::addEntry(const __Key& key)
{
    __Tp* entry = new __Tp(routingUnit_);
    pair<typename map<__Key,__Tp*>::iterator, bool> p =
      base_.insert(map_make_pair<__Key,__Tp*>(key,entry));
    if (!p.second)
      { // there is already an entry with this key, replace it
        __Tp* oldEntry = static_cast<__Tp*>((p.first)->second);
        delete oldEntry;
        base_.erase(p.first);
        p = base_.insert(map_make_pair<__Key,__Tp*>(key,entry));
        assert(p.second);
      }
    return entry;
}

/*! \returns entry which key match the given key if any, 0 otherwise
    If there are more than one entry for the given key, return the first
    entry found. */
template<class __Key,class __Tp,class __PathItemData> __Tp*
CEASRoutingUnitIB<__Key,__Tp*,__PathItemData>::findEntry(const __Key& key)
{
    typename map<__Key,__Tp*>::iterator i = base_.find(key);
    if (i != base_.end())
      // found matching entry
      return static_cast<__Tp*>(i->second);
    return 0;
}

/*! \returns entry which match the given key. If no entry is found, create one.
    If there are more than one entry for the given key, return the first
    entry found. */
template<class __Key,class __Tp,class __PathItemData> __Tp*
CEASRoutingUnitIB<__Key,__Tp*,__PathItemData>::getEntry(const __Key& key)
{
    __Tp* entry = findEntry(key);
    if (!entry) entry = addEntry(key);
    return entry;
}

#endif
