/*
 * Copyright (c) 2008 Q2S NTNU, Trondheim, Norway
 * All rights reserved.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation;
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 * Port of the Swarm module developed by O. Wittner for ns-2.30.
 * Author: Laurent Paquereau <laurent.paquereau@q2s.ntnu.no>
 *
 * $Header$
 */
 
#include "ceas-temperature.h"
#include <math.h>
#include <assert.h>
#include <stdio.h>
#include <stdlib.h>

//! Update temperature
double
CEASTemperature::update(double L,double beta,double rho)
{
    fprintf(stderr,"CEASTemperature::%s: CEASTemperature is a generic " \
            "class. Use a subclass.\n",__FUNCTION__);
    abort();
    return 0.0;
}

double
CEASTemperatureHyp::update(double L,double beta,double rho)
{
    double L_on_gamma,H;
    // increment t and update gamma
    if (++t_ == 1)
      // first sample
      // NB: using the general formula for computing gamma theoratically keeps
      // gamma unchanged, however it may be altered (doubleing point operation)
      // hence this particular case
      gamma_ = -L/log(rho);
    else
      {
        L_on_gamma = L/gamma_;
        H = exp(-L_on_gamma );
        double old_gamma = gamma_;
        gamma_ = (b_+H*L)/(a_+H*(1+L_on_gamma)-rho*(1-pow(beta,t_))/(1-beta));
        if (gamma_<=0) 
          {
            gamma_ = old_gamma;
            fprintf(stderr,"CEASTemperatureHyp::%s: Keeping old gamma (%d)\n",
                    __FUNCTION__,t_);
          }
      }
    // update a and b
    L_on_gamma = L/gamma_;
    H = exp(-L_on_gamma );
    a_ = beta*(a_+(1+L_on_gamma)*H);
    b_ = beta*(b_+L*H);
    return gamma_;
}

double
CEASTemperatureLin::update(double L,double beta,double rho)
{
    double L_on_gamma,H;
    // increment t and update gamma
    if (++t_ == 1)
      // first sample
      // NB: using the general formula for computing gamma theoratically keeps
      // gamma unchanged, however it may be altered (floating point operation)
      // hence this particular case
      gamma_ = -L/log(rho);
    else
      {
        L_on_gamma = L/gamma_;
        double gamma_min = (rho*(1-pow(beta,t_))/(1-beta)-1-a_)/b_;
        double a = a_;
        double b = b_;
        if(L_on_gamma < 2)
          {
            L_on_gamma = 2;
            gamma_ = L/2;
            double aa = -exp(-2);
            double bb = 4*exp(-2)/L;
            double m = (1-beta)/(1-pow(beta,t_));
            double y = m*(aa*b_-a_*bb)/(b_);
            if (y > rho)
              {
                a = 0;
                b = 0;
              }
          }
        H = exp(-L_on_gamma );
        assert(H!=HUGE_VAL && H!=-HUGE_VAL);
        gamma_ = -(a+H*(1-L_on_gamma)-rho*(1-pow(beta,t_))/(1-beta))/
                 (b+H*L_on_gamma/gamma_);
        if (gamma_ < gamma_min) 
          {
            gamma_ = gamma_min;
            L = 4*exp(-2)*gamma_/(rho*(1-pow(beta,t_))/(1-beta)+exp(-2));
          }
        else
          {
            a_ = a;
            b_ = b;
          }
      }
    // update a and b
    L_on_gamma = L/gamma_;
    H = exp(-L_on_gamma );
    a_ = beta*(a_+(1-L_on_gamma)*H);
    b_ = beta*(b_+L_on_gamma/gamma_*H);
    return gamma_;
}

