/*
 * Copyright (c) 2008 Q2S NTNU, Trondheim, Norway
 * All rights reserved.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation;
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 * Port of the Swarm module developed by O. Wittner for ns-2.30.
 * Author: Laurent Paquereau <laurent.paquereau@q2s.ntnu.no>
 *
 * $Header$
 */
 
#ifndef _ceas_temperature_h
#define _ceas_temperature_h

enum CEASTemperature_t {
  CEAS_TEMPERATURE_HYPERBOLIC_APPROX = 0,
  CEAS_TEMPERATURE_LINEAR_APPROX = 1
};

//! Base class
class CEASTemperature {
  public:
    CEASTemperature() : gamma_(0),a_(0.0),b_(0.0),t_(0) {};
    inline double& gamma() {return gamma_ ;};
    virtual double a() {return a_;}
    virtual double b() {return b_;}
    virtual double update(double L, double beta, double rho);
  protected:
    double gamma_; //!< actual temperature
    double a_; //!< auto-regression parameter
    double b_; //!< auto-regression parameter
    unsigned int t_; //!< number of iterations
};

//! Use hyperbolic approximation
class CEASTemperatureHyp : public CEASTemperature {
  public:
    CEASTemperatureHyp() : CEASTemperature() {};
    virtual double update(double L, double beta, double rho);
};

//! Use linear approximation
class CEASTemperatureLin : public CEASTemperature {
  public:
    CEASTemperatureLin() : CEASTemperature() {};
    virtual double update(double L, double beta, double rho);
};

#endif

