/*
 * Copyright (c) 2008 Q2S NTNU, Trondheim, Norway
 * All rights reserved.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation;
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 * Port of the Swarm module developed by O. Wittner for ns-2.30.
 * Author: Laurent Paquereau <laurent.paquereau@q2s.ntnu.no>
 *
 * $Header$
 */

#ifndef _ceas_utils_h
#define _ceas_utils_h

#include <map>
#include <stdio.h>

/*! Specialized map; value_type is FILE* */
template<class _Key> class map_file : private map<_Key,FILE*> {
  public:
    map_file() : map<_Key,FILE*>() {}
    FILE*& operator[](const _Key& k)
    {
        typename map<_Key,FILE*>::iterator i = lower_bound(k);
        if (i == map<_Key,FILE*>::end() ||
            map<_Key,FILE*>::key_comp()(k, (*i).first))
          i = map<_Key,FILE*>::insert(
            i,pair<const _Key,FILE*>(k,NULL));
          return (*i).second;
    }
};

/*! Specialized map; value_type is double and it is possible to define a
    default_value */
template<class _Key> class map_double : private map<_Key,double> {
  public:
    map_double() : map<_Key,double>(),default_value_(0.0) {}
    map_double(double default_value)
      : map<_Key,double>() , default_value_(default_value) {}
    double& operator[](const _Key& k)
    {
        typename map<_Key,double>::iterator i = lower_bound(k);
        if (i == map<_Key,double>::end() ||
            map<_Key,double>::key_comp()(k, (*i).first))
          i = map<_Key,double>::insert(
            i,pair<const _Key,double>(k,default_value_));
          return (*i).second;
    }
  protected:
    double default_value_;
};

/*! Specialized map; value_type is unsigned int and it is possible to define a
    default_value */
template<class _Key> class map_uint : private map<_Key,unsigned int> {
  public:
    map_uint() : map<_Key,unsigned int>(),default_value_(0) {}
    map_uint(unsigned int default_value)
      : map<_Key,unsigned int>() , default_value_(default_value) {}
    unsigned int& operator[](const _Key& k)
    {
        typename map<_Key,unsigned int>::iterator i = lower_bound(k);
        if (i == map<_Key,unsigned int>::end() ||
            map<_Key,unsigned int>::key_comp()(k, (*i).first))
          i = map<_Key,unsigned int>::insert(
            i,pair<const _Key,unsigned int>(k,default_value_));
          return (*i).second;
    }
  protected:
    unsigned int default_value_;
};

#endif
