#ifndef _plain_ceas_forwarding_unit_h
#define _plain_ceas_forwarding_unit_h

#include "ceas-forwarding-unit.h"
#include "plain-ceas-network-layer-unit.h"



class Plain_CEASForwardingUnit : public CEASForwardingUnit<> {
	public:
	int oneHopMemory;
	//! Constructor
	Plain_CEASForwardingUnit(Plain_CEASNetworkLayerUnit* networkLayerUnit)
	  : CEASForwardingUnit<>(networkLayerUnit) {oneHopMemory=0; }
	void recv(Packet* p, Handler* h);
};

void
Plain_CEASForwardingUnit::recv(Packet* p, Handler* h)
{
	
	hdr_cmn* ch = hdr_cmn::access(p);	
	hdr_ip* ih = hdr_ip::access(p);
	hdr_tcp* tp = hdr_tcp::access(p);
		
	

	if(ch->direction() == hdr_cmn::UP)
	{
		// Incomming Packet

		/*if(tp->reason()==TCP_REASON_DUPACK)
			printf("%g, DUPACK\n", tp->ts());*/
		//printf("DUPACK:\n",tp->seqno());
			
		// Log trace info
		if(logger()); // Need to log
		networkLayerUnit()->sendUp(p,h);
		return;
	}


	
	if(ch->direction() == hdr_cmn::DOWN)
	{
		// Outgoing Packet
				
		vector<pair<NeighbourEntry*, double> > neighbours; 	//List of neighbouring nodes with their respective pheromone level
		vector<pair<NeighbourEntry*,double> > NewNeighbours;	//List of neighbouring nodes with their respective pheromone level except the one that forward the packet
		NeighbourEntry* next; //Next node to which packet is to be forwarded.

		// Build list of neighbours associated with the cumulative pherolevel		
		networkLayerUnit()->getNeighboursCumulativePheroLevel(ih->flowid(),
								ih->saddr(),
								ih->daddr(),
								neighbours);
		
	
		if(neighbours.empty())
		{
			// No neighbour to forward packet, therefore drop packet
			networkLayerUnit()->drop(p, DROP_RTR_NO_ROUTE);
			return;
		}	
	

		// Remove forwarded node from the neighbour list
		vector<pair<NeighbourEntry*,double> >::iterator i = neighbours.begin();

		if(oneHopMemory && (neighbours.size() > 1)) {
			for(;i != neighbours.end(); i++)
			{
				if(i->first->getAddress() != ch->prev_hop())
				{
					NewNeighbours.push_back(
				          pair<NeighbourEntry*,double>(i->first,i->second));
					
				}
			}
			next = networkLayerUnit()->selectRandomNeighbour(NewNeighbours);

		}
		else
		    	next = networkLayerUnit()->selectRandomNeighbour(neighbours);
		

		// Forward the packet
		networkLayerUnit()->sendDown(p,h,next);
		return;


	}
	
	

	
}

#endif
