#### Automatic detection of the path to CEAS sources ####

set nsPath [exec which ns]
set CEASPath [string replace $nsPath [expr [string length $nsPath] - 6] [expr [string length $nsPath] -1] CEAS]
set rng [new RNG]
$rng seed 5

set e [new RandomVariable/Uniform]
$e set min_ 10
$e set max_ 30
$e use-rng $rng

#### Parameters ####

set param(DEBUG_ON)			0
set param(TRACE_ON)			0
set param(NAMTRACE_ON)			0
set param(Topology)			"uninett_oct.tcl"

set param(StopToContinue)		0
set param(Finish)			1800

set param(ProgressPeriod)   		1;#[expr $param(Finish) / 1000 ]  		;# Time periode for progress report
if { $param(ProgressPeriod) < 1  } {
    set param(ProgressPeriod) 1
}

set param(Seed)				179324
set param(BinSize)			100					;# No of Samples in bins when averaging
set param(SampleFreq)			[expr 2000000.0/1000 ];#[expr 2000000.0 / $param(Finish)]	;# No of Samples per simulated second

set param(InitRate)			500					;# No of Swarm agent per second to generate during initialization phase
set param(GenRate)			80.0					;# No of Ants per second to generate (low rate)
set param(ExplorerRate)			4					;# No of explorer swarm agents per second to generate


set param(DataRate)			200Mb;

set param(TTL)				60					;# Initial TTL value
set param(PacketSize)			1500

set param(ProcDelayAnt)			0.000					;# CEAS agent processing delay in nodes
set param(ProcDelayData)		0.000					;# Data packet processing delay in nodes

set param(Beta)				0.98					;# "Evaporation" rate (path memory)
set param(Rho)				0.01					;# Search focus

# Trace files
set param(TraceFile)			"trace.tr"
set param(NamTraceFile)			"namtrace.nam"

set param(PMInterval)			1					;# TCP performance monitor interval in seconds

# Matrix of connections, a row is "S.no species-source-destination-initPhaseStart-nitPhaseEnd-startTime" TCP simplex
set param(Connections)			"\
1 1-45-51-0-50-100-2Gb\
2 1-51-45-0-50-100-2Gb\
3 2-51-47-0-50-100-50Mb\
4 2-47-51-0-50-100-50Mb\
5 4-51-25-50-100-150-1.7Gb\
6 4-25-51-50-100-150-1.7Gb\
7 5-51-4-0-50-100-2Gb\
8 5-4-51-0-50-100-2Gb\
"

set param(LongFailures)  	"\
1 51-50-down\
2 50-51-down\
3 56-36-down\
4 36-56-down\
5 19-42-down\
6 42-19-down\
"

set param(OptFailure)		"\
1 34-39-down\
2 39-34-down\
3 39-15-down\
4 15-39-down\
5 15-12-down\
6 12-15-down\
7 12-2-down\
8 2-12-down\
9 2-35-down\
10 35-2-down\
"
set param(RouteFailure)		"\
1 5-33-down\
2 33-5-down\
3 5-4-down\
4 4-5-down\
5 5-46-down\
6 46-5-down\
7 5-41-down\
8 41-5-down\
9 5-1-down\
10 1-5-down\
11 18-53-down\
12 53-18-down\
13 18-32-down\
14 32-18-down\
15 18-48-down\
16 48-18-down\
17 18-17-down\
18 17-18-down\
"

set param(HighFailure)		"\
1 52-54-0.8\
2 54-52-0.8\
3 34-35-0.5\
4 35-34-0.5\
5 51-1-1\
6 1-51-1\
"
set param(Failures) 		"\
1 650-4-5-down\
2 650-5-4-down\
"

#################################################
# P R O C E D U R E S    A N D    C L A S S E S #
#################################################

#### Scenario Classes ####

# Base class
source $CEASPath/libs/common/src/ceas-scenario.tcl

# Load Dynamic library
dynlibload ceasplain

# Current Scenario
Class TCP_scenario -superclass CEAS_scenario

TCP_scenario instproc init { param } {
	# Constructor
	$self instvar SimID_ 
	
	# Add SimID to param array
	array set param_tmp $param
	$self get_simid
	set param_tmp(Seed) [expr $param_tmp(Seed) + $SimID_]
	set param [array get param_tmp]

	# Add scenario title to parm-array and run parent class constructor
	$self next "$param Scenario TCP_scenario"
	
	
}

TCP_scenario instproc init_simulation {} {
	$self next
	$self instvar ns_ param_ pm_f_ 
	# Configure CEAS
	$ns_ CEAS-config -elite-selection       all \
			 -routing-loops		allow \
			 -log-bin-size		$param_(BinSize) \
			 -log-sample-freq	$param_(SampleFreq) \
			 -log-only 1	
			 
			 

	$ns_ dump-CEAS-config
	
	# For TCP performance log file
	array set connection $param_(Connections)
			set i [array startsearch connection ]
			while { [array anymore connection $i]} {
				set cx [array nextelement connection $i]
				set species [lindex [split $connection($cx) "-"] 0]
				if { ! [info exist pm_f_($species)]} {
					set pm_f_($species) [open pmtrace_id_$species.txt  w]
					puts "File: $species"
				}

			}
			array donesearch connection $i
}

TCP_scenario instproc init_components {} {
	# Prepare misc. component required in simulation
	$self instvar gen_ ns_ node_ param_ gen_expl_ ceas_network_layer_unit_ ceasgen_data_ tcp_
	
	# Create random generator (common for all swarm systems)
	set randomgen [new RNG]
	$randomgen seed $param_(Seed)
	
	# Set Queue Length
	set i [array startsearch node_ ]
	while { [array anymore node_ $i] } {
	    set ix [array nextelement node_ $i]
	    set j [array startsearch node_]
	    while { [array anymore node_ $j] } {
		set jx [array nextelement node_ $j]
		set link [$ns_ link $node_($ix) $node_($jx)]
		if { $link != "" } {
			# Link exists between nodes. Set queue limit
			$ns_ queue-limit $node_($ix) $node_($jx) 50
		}
	    }
	    array donesearch node_ $j
	}
	array donesearch node_ $i

	# Get array of connections
	array set connection $param_(Connections)
	
	# Create and init swarm generators, two (normal + explorer) per connections
	set i [ array startsearch connection ]	
	while { [array anymore connection $i] } {
		set c [array nextelement connection $i]
		set species [lindex [split $connection($c) "-"] 0]
		set src [lindex [split $connection($c) "-"] 1]
		set dst [lindex [split $connection($c) "-"] 2]
		set dRate [lindex [split $connection($c) "-"] 6]
		puts "Species: $species, Source: $src, Destination: $dst"		
		
		# Normal ant generator
		set gen_($species,$src,$dst) [$ceas_network_layer_unit_($src) create-generator CEAS/Plain]
		$gen_($species,$src,$dst) set-type CBR
		$gen_($species,$src,$dst) set-rate $param_(InitRate)
		$gen_($species,$src,$dst) set-species $species
		$gen_($species,$src,$dst) set-beta $param_(Beta)
		$gen_($species,$src,$dst) set ttl_ $param_(TTL)
		$gen_($species,$src,$dst) set-dst-address [$node_($dst) node-addr]		

		# Explorer Ant generator
		set gen_expl_($species,$src,$dst) [$ceas_network_layer_unit_($src) create-generator CEAS/Plain]
		$gen_expl_($species,$src,$dst) set-type CBR
		$gen_expl_($species,$src,$dst) set-dst-address [$node_($dst) node-addr]
		$gen_expl_($species,$src,$dst) set-behaviour explorer
		$gen_expl_($species,$src,$dst) set-rate $param_(ExplorerRate)
		$gen_expl_($species,$src,$dst) set-species $species
		$gen_expl_($species,$src,$dst) set-beta $param_(Beta)
		$gen_expl_($species,$src,$dst) set ttl_ $param_(TTL)

		# Data packet generator (Constant Bit rate over TCP) Simplex only
		if {![info exist ceasgen_data_($species)]} {
			puts "Creating TCP session from $src to $dst . . . "
			set ceasgen_data_($species) [new Application/Traffic/CBR]
			set tcp_($species) [new Agent/TCP/Reno]    
			set sink [new Agent/TCPSink/DelAck]
			$ns_ attach-agent $node_($src) $tcp_($species)
			$ns_ attach-agent $node_($dst) $sink
			$ceasgen_data_($species) attach-agent $tcp_($species)
			$ns_ connect $tcp_($species) $sink
			
			$tcp_($species) set fid_ $species			;# Flow id
			$sink set fid_ $species					;# Flow id
			$tcp_($species) set ttl_ $param_(TTL)			;# TTL
			$tcp_($species) set numdupacks_ 3
			$sink set ttl_ $param_(TTL)
			$sink listen
			$tcp_($species) set window_ 1000
			
			$tcp_($species) set packetSize_ $param_(PacketSize)
			 
			$ceasgen_data_($species) set rate_ $dRate
			$ceasgen_data_($species) set packetSize_ $param_(PacketSize)  
		}
	}
	array donesearch connection $i
	
	# Load the Background Traffic
	#$self loadTraffic
	
	# For each node ...
	set n_idx [array startsearch node_]
	while { [array anymore node_ $n_idx]} {
	    set n [array nextelement node_ $n_idx]	
	    
	    # Following line is needed when ForwardingUnit installed on a node
	    [$node_($n) network-layer-manager] default-target $ceas_network_layer_unit_($n)
	
	    
	    # Set the random generator
	    $ceas_network_layer_unit_($n) use-rng $randomgen
		
	    # Set swarm agent processing time
	    $ceas_network_layer_unit_($n) set-service-time $param_(ProcDelayAnt)
	    $ceas_network_layer_unit_($n) set-service-time-data $param_(ProcDelayData)
		
	    # Atttach logger
	    $ceas_network_layer_unit_($n) attach-logger
	
	}
	array donesearch node_ $n_idx
	
}

TCP_scenario instproc schedule_events {} {
	# List events to take place during the sim. scenario
	# Schedule events
	$self next			;# Call parent clas instproc
	$self instvar ns_ param_ node_ numnodes_ colors_ gen_ gen_expl_ ceasgen_data_
	#$self recNam 100
	# Get array of connections
	array set connection $param_(Connections)
	# Schedule start of swarm generators
	set i [array startsearch connection]
	while { [array anymore connection $i] } {
		set c [array nextelement connection $i]
		set species [lindex [split $connection($c) "-"] 0]
	    	set src [lindex [split $connection($c) "-"] 1]
	    	set dst [lindex [split $connection($c) "-"] 2]
		set iStart [lindex [split $connection($c) "-"] 3]
		set iEnd [lindex [split $connection($c) "-"] 4]
		set sTime [lindex [split $connection($c) "-"] 5]
		$ns_ at $iStart "$gen_($species,$src,$dst) set-behaviour explorer"
	    	$ns_ at $iStart "$gen_($species,$src,$dst) start"
		
		$ns_ at $iEnd "$gen_($species,$src,$dst) set-behaviour normal"
	    	$ns_ at $iEnd "$gen_($species,$src,$dst) set-rate $param_(GenRate)"	
	    	
		$ns_ at $iStart "$gen_expl_($species,$src,$dst) start"
		
		if { ! [info exist tcp_started($species)] } {
		    # Start data generator
		   $ns_ at $sTime "$ceasgen_data_($species) start"
		   
		   # Start TCP Performance monitor
		   $ns_ at $sTime "$self set tcp_sent_($species) 0; $self set tcp_retrans_($species) 0"	 
		   $ns_ at $sTime "$self tcp-performance $species $param_(PMInterval)"
		   set tcp_started($species) yes
		    
	    }
	}
	array donesearch connection $i
	# Schedule failure
	#$self failure
}

TCP_scenario instproc tcp-performance { species interval } {
	$self instvar ns_ tcp_ tcp_sent_ tcp_retrans_ pm_f_
	
	# Calculate goodput and throughput during interval
	set sent [expr [$tcp_($species) set ndatabytes_] - $tcp_sent_($species) ]
	set retrans [expr [$tcp_($species) set nrexmitbytes_] - $tcp_retrans_($species) ]
	set goodput [expr ($sent - $retrans )/ $interval ]
	set throughput [expr $sent / $interval]

	set tcp_sent_($species) [$tcp_($species) set ndatabytes_]
	set tcp_retrans_($species) [$tcp_($species) set nrexmitbytes_]

	# Log Goodput and throughputs
	 puts $pm_f_($species) "[ $ns_ now ] sent $sent ([ $tcp_($species) set ndatabytes_ ]) retrans $retrans ([ $tcp_($species) set nrexmitbytes_ ]) goodput $goodput ( backoff [ $tcp_($species) set backoff_ ] ) throughput $throughput"
	set sentMb [expr ($sent*8)/1000000]
	puts  "TCP: $species [ $ns_ now ] sent $sent : $sentMb Mb Retransmitred $retrans ([ $tcp_($species) set nrexmitbytes_ ]) goodput $goodput [expr ($goodput*8)/1000000] Mb Cwnd [ $tcp_($species) set cwnd_]"
	

    	# Reschedule goodput calculation
    	$ns_ at [expr [ $ns_ now ] + $interval ] "$self tcp-performance $species $interval"
}

TCP_scenario instproc finish {} {
	# Close TCP Performance log file
	$self instvar pm_f_ param_
	array set connection $param_(Connections)
			set i [array startsearch connection ]
			while { [array anymore connection $i]} {
				set cx [array nextelement connection $i]
				set species [lindex [split $connection($cx) "-"] 0]
				if { ! [info exist pm_f_($species)]} {
					close $pm_f_($species)
				}

			}
			array donesearch connection $i	
	
	
	$self next
}

TCP_scenario instproc recNam {sTime } {
	$self instvar nam_f_ ns_ node_ param_
	
	if { $param_(NAMTRACE_ON) } {
	$ns_ at $sTime "$ns_ namtrace-queue $node_(3) $node_(4) $nam_f_"
	$ns_ at $sTime "$ns_ namtrace-queue $node_(4) $node_(3) $nam_f_"
        $ns_ at $sTime "$ns_ namtrace-queue $node_(11) $node_(5) $nam_f_"
        $ns_ at $sTime "$ns_ namtrace-queue $node_(5) $node_(11) $nam_f_"
        $ns_ at $sTime "$ns_ namtrace-queue $node_(3) $node_(10) $nam_f_"
   	$ns_ at $sTime "$ns_ namtrace-queue $node_(10) $node_(3) $nam_f_"
   	$ns_ at $sTime "$ns_ namtrace-queue $node_(9) $node_(5) $nam_f_"
   	$ns_ at $sTime "$ns_ namtrace-queue $node_(5) $node_(9) $nam_f_"	
	}
}
TCP_scenario instproc failure {} {
	# Add Failure events
	$self instvar ns_ node_ param_

	# Get array of Failures
	array set failure $param_(Failures)

	set i [ array startsearch failure]
	while { [array anymore failure $i ] } {
		set c [array nextelement failure $i]
		set ndx_time [lindex [split $failure($c) "-"] 0]
		set n1 [ lindex [ split $failure($c) "-"] 1]
		set n2 [ lindex [ split $failure($c) "-"] 2]
		set state [ lindex [ split $failure($c) "-"] 3]
		
		$ns_ rtmodel-at $ndx_time $state $node_($n1) $node_($n2)
		puts " At $ndx_time : link $n1 --> $n2 $state "

	}
	array donesearch failure $i
	
}
###########
# M A I N #  
###########

# Include only swarm packets and IP packets
#remove-all-packet-headers 
#add-packet-header IP CEAS

#
# Create simulator instance
#
set ns [new Simulator] 

# Create scenario
new TCP_scenario [array get param]

# Run simulation
$ns run
